
/*]  Copyright (c) 2009-2010, Charles McGarvey  [**************************
**]  All rights reserved.
*
* vi:ts=4 sw=4 tw=75
*
* Distributable under the terms and conditions of the 2-clause BSD license;
* see the file COPYING for a complete text of the license.
*
**************************************************************************/

#ifndef _MAIN_HH_
#define _MAIN_HH_

/**
 * \file Main.hh
 * This is where all the fun begins.
 */

#include <iostream>
#include <string>

#include <boost/shared_ptr.hpp>

#include <moof/application.hh>
#include <moof/dispatcher.hh>
#include <moof/math.hh>
#include <moof/timer.hh>

#include "yoink.hh"


class Main : public moof::application
{
public:

	Main(moof::settings& settings);

	void update(moof::scalar t, moof::scalar dt);
	void draw(moof::scalar alpha) const;
	void handle_event(const moof::event& event);

	static std::string search_paths();
	static std::string config_paths();

	static void print_usage();
	static void print_info(int argc, char* argv[]);

private:

	/**
	 * Set OpenGL to a state we can know and depend on.
	 */
	static void setup_opengl();

	static void print_option(const std::string& option, bool enabled);

	moof::dispatcher::handle	video_reloaded_;
	moof::timer					hotload_timer_;

	::yoink	yoink;
};


#endif // _MAIN_HH_

